﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using System.IO;

namespace HeuristicLab.Problems.BinPacking2D {
  public static class BPPORLIBParser {
    public static BPPData Parse(string file) {
      using (Stream stream = new FileStream(file, FileMode.Open, FileAccess.Read)) {
        return Parse(stream);
      }
    }

    public static void Export(BPPData instance, string file) {
      using (Stream stream = new FileStream(file, FileMode.Open, FileAccess.Read)) {
        Export(instance, stream);
      }
    }

    /// <summary>
    /// Reads from the given stream data which is expected to be in the BPP ORLIB format.
    /// </summary>
    /// <remarks>
    /// The stream is not closed or disposed. The caller has to take care of that.
    /// </remarks>
    /// <param name="stream">The stream to read data from.</param>
    /// <returns>The parsed problem instance</returns>
    public static BPPData Parse(Stream stream) {
      using (var reader = new StreamReader(stream)) {
        var instance = new BPPData();
        var delim = new char[] { ' ', '\t' };
        var problemClass = reader.ReadLine().Split(delim, StringSplitOptions.RemoveEmptyEntries);

        var tok = reader.ReadLine().Split(delim, StringSplitOptions.RemoveEmptyEntries);
        var numItems = int.Parse(tok[0]);
        //Numbering of instances..
        reader.ReadLine();
        tok = reader.ReadLine().Split(delim, StringSplitOptions.RemoveEmptyEntries);
        instance.BinShape = new PackingShape(int.Parse(tok[0]), int.Parse(tok[1]));
        instance.Items = new PackingItem[numItems];

        for (int k = 0; k < numItems; k++) {
          if (reader.EndOfStream) throw new InvalidDataException("Unexpected End of Stream (at line " + k + ").");
          var valLine = reader.ReadLine();
          while (String.IsNullOrWhiteSpace(valLine)) valLine = reader.ReadLine();
          tok = valLine.Split(delim, StringSplitOptions.RemoveEmptyEntries);
          instance.Items[k] = new PackingItem(int.Parse(tok[0]), int.Parse(tok[1]), instance.BinShape) { Material = 1, Weight = 1.0 }; // these problem instances do not specify weights and materials
        }

        int problemClassNr = -1;
        if (int.TryParse(problemClass[0], out problemClassNr)) {
          instance.Name = "dbpp_class_0" + problemClassNr;
          if (problemClassNr >= 1 && problemClassNr <= 5)
            instance.Description = "Proposed by Berkey and Wang.";
          else if (problemClassNr >= 6 && problemClassNr <= 10)
            instance.Description = "Proposed by Martello and Vigo.";
        } else {
          instance.Name = "Unknown";
          instance.Description = "<Missing>";
        }

        return instance;
      }
    }

    public static void Export(BPPData instance, Stream stream) {
      var nameParts = instance.Name.Split('_');
      int problemClassNr = int.Parse(nameParts[2]);

      using (var writer = new StreamWriter(stream)) {
        writer.WriteLine(String.Format("{0,-5}               PROBLEM CLASS ({1})", problemClassNr, instance.Description));
        writer.WriteLine(String.Format("{0,-5} {1,-5}   WBIN,HBIN", instance.BinShape.Width, instance.BinShape.Height));
        for (int i = 0; i < instance.NumItems; i++) {
          if (i == 0)
            writer.WriteLine("{0,-5} {1,-5}   W(I),H(I),I=1,...,N", instance.Items[i].Width, instance.Items[i].Height);
          else
            writer.WriteLine("{0,-5} {1,-5}", instance.Items[i].Width, instance.Items[i].Height);

        }
        writer.Flush();
      }
    }

  }
}
